/*
 *  np_main.h
 *  NtileCodePredictor
 *
 *  Created by Naoto Morikawa on 2/19/06.
 *  Copyright (C) 2007 Naoto Morikawa. All rights reserved.
 *
 *  2007-03-05 version 0.1 release
 */

#define STDOUT stdout
#define STDERR stdout
#define DEBUG 1

#define SUCCESS 0
#define FAIL 1
#define YES 0
#define NONO 1

#define PRED_SCORE(aa_ind, tbl_ct) pt_protein->aa[(aa_ind)].score_tbl\
				[pt_protein->aa[(aa_ind)].prediction[(tbl_ct)]] 
#define PRED_CODE(aa_ind, i) pt_protein->aa[(aa_ind)].prediction[(i)]
#define FLOAT_NEQ(fdata) ((fdata) * (fdata) >  0.0001)
#define MAX_CODE_SIZE	10000		// for load_code_file()

// comp_result
#define R_SUCCESS	0
#define R_FAIL		1
#define R_CONTAINED	2
#define R_NOPREDICTION	3

/* Amino acids */
#define L_GLY 0			/* glycine, G */

#define L_ALA 1			/* alanine, A */
#define L_VAL 2			/* valine, V */
#define L_PHE 3			/* phenylalanine, F */
#define L_PRO 4			/* proline, P */
#define L_HYP 23		/* Hydroxyproline, - */
#define L_MET 5			/* methionine, M */
#define L_ILE 6			/* isoleucine, I */
#define L_LEU 7			/* leucine, L */

#define L_ASP 8			/* aspartic acid, D */
#define L_GLU 9			/* glutamic acid, E */
#define L_LYS 10		/* lysine, K */
#define L_ARG 11		/* arginine, R */

#define L_SER 12		/* serine, S */
#define L_THR 13		/* threonine, T */
#define L_TYR 14		/* tyrosine, Y */
#define L_HIS 15		/* histidine, H */
#define L_CYS 16		/* cysteine, C */
#define L_ASN 17		/* asparagine, N */
#define L_GLN 18		/* glutamine, Q */
#define L_TRP 19		/* tryptophan, W */

#define L_ASX 20		/* ASP/ASN ambiguous, B */
#define L_GLX 21		/* GLU/GLN ambiguous, Z */
#define L_UNK 22		/* unknown, = or X */
#define L_ELSE 24		/* '?' */

/* Score table entries */
#define SCORE_TABLE_SIZE 35	// 35 = 32 + 3 <=> 2**5 + '-' + '?' + else

#define ST_0		0	// DDDDD
#define ST_1		1	// DDDDU
#define ST_2		2
#define ST_3		3
#define ST_4		4
#define ST_5		5
#define ST_6		6
#define ST_7		7
#define ST_8		8
#define ST_9		9
#define ST_A		10
#define ST_B		11
#define ST_C		12
#define ST_D		13
#define ST_E		14
#define ST_F		15
#define ST_G		16
#define ST_H		17
#define ST_I		18
#define ST_J		19
#define ST_K		20
#define ST_L		21
#define ST_M		22
#define ST_N		23
#define ST_O		24
#define ST_P		25
#define ST_Q		26
#define ST_R		27
#define ST_S		28
#define ST_T		29
#define ST_U		30	// UUUUD
#define ST_V		31	// UUUUU

#define ST_UNK		32	// '?'
#define ST_UNDEF	33	// '-'
#define ST_INV		34	// else


typedef enum boolean {
	TRUE,
	FALSE
} BOOLEAN;


/**
 * protein
 *   |
 *   +------+-  chain[1].aa_head ->  aa[1]
 *          |                          |
 *          |                        aa[2]
 *          |                          |
 *          |
 *          |
 *          |                          |
 *          |-  chain[1].aa_tail -> aa[k]
 *          |                          |
 *          +-  chain[2].aa_head -> aa[k+1]
 *          |                          |
 *          |                       aa[k+2]
 *          |                          |
 *
 */

/**
 * struct np_amino_acid - is an amino acid
 * @name  : amino acid name
 * @chain_id   : chain identifier
 * @score_tbl  : score table of Ntile codes
 * @num_valid_scores: total score (hit count)
 * @actual     : computed Ntile code (to be loaded from ".code" files)
 * @prediction : sorted Ntile codes (in descending order based on the score)
 * @num_topscorer: number of the Ntile code with the top score
 * @comp_result :  R_SUCCESS / R_FAIL / R_CONTAINED / R_NOPREDICTION
 * @accuracy   : quality of the prediction
 */
struct np_amino_acid {
	int name;
	char chain_id;
	float score_tbl[SCORE_TABLE_SIZE];
	float num_valid_scores;
	int actual;
	int prediction[SCORE_TABLE_SIZE];
	int num_topscorer;
	int comp_result;
	float accuracy;
};

/**
 * struct np_chain - is an amino acid sequence
 * @aa_head  : the first amino acid of the sequence
 * @aa_tail  : the last amino acid of the sequence
 * @aa_num   : number of amino acids
 * @id       : chain identifier
 * @coverage : number of predicted amino acids / number of all amino acids
 */
struct np_chain {
	int aa_head;
	int aa_tail;
	int aa_num;
	int id;
	float coverage;
};

/**
 * struct np_protein - is the input amino acid sequences
 * @chain_head: the first amino acid sequence
 * @chain_tail: the last amino acid sequence
 * @chain_num : number of sequences
 * @aa_num    : number of amino acids
 * @chain     : chains of the protein
 * @aa        : amino acids of the protein
 * @filename  : the name of the input protein file
 * @milt_ct               : number of amino acids with multiple top scoreres
 * @success_ct            : number of successful prediction
 * @contained_ct          : number of amino acids whose actual code is one of 
 *                          the multiple top scoreres
 * @hit_ct                : number of predicted amino acids
 * @coverage              : #{predicted amino acids} / #{all amino acids}
 * @ave_accuracy          : average accuracy
 * @mult_candidate_rate   : mult_ct / (#aa - #chain * 4)
 * @success_rate          : success_ct / (#aa - #chain * 4)
 * @extended_success_rate : (success_ct + contained_ct) / (#aa - #chain * 4)
 */
struct np_protein {
	int chain_head;
	int chain_tail;
	int chain_num;
	int aa_num;
	struct np_chain * chain;
	struct np_amino_acid * aa;
	const char * filename;
	
	// stats
	int mult_ct;
	int success_ct;
	int contained_ct;
	int hit_ct;
	float coverage;
	float ave_accuracy;
	float mult_candidate_rate;
	float success_rate;
	float extended_success_rate;
};

/**
 * struct np_fragment - is a fragment of amino acids
 * @chn_ct : chain identifier
 * @aa_ct  : initial amino acid
 * @length : length of the fragment
 */
struct np_fragment {
	int chn_ct;
	int aa_ct;
	int length;
};

/**
 * struct np_settings - is the my_settings of the software
 * @search_table_full: enable search of "frag_code5_full.tbl" table
 * @search_table5    : enable search of "frag_code5.tbl" table
 * @search_table7    : enable search of "frag_code5.tbl" table
 * @compare_enabled  : enable comparison between prediction and actual code
 * @create_view_file : enable creation of the ".code" file
 * @tiles_per_res    : considered fragment length
 */
struct np_settings {
	BOOLEAN search_table_full;
	BOOLEAN search_table5;
	BOOLEAN search_table7;
	BOOLEAN search_table9;
	BOOLEAN compare_enabled;
	BOOLEAN create_view_file;
	float tiles_per_res;
};
